/* -- File Info --
 * org.cpp - Implementation file
 *
 * Simply a public domain ORG player / library.
 * Do what you wish with it. Providing credit would be cool but I just don't see many people wanting to do that so meh.
 *
 * To note, this code is PD. You may copy it, redistribute it, and use it as you wish, unrestricted.
 *   ~nmn
 */

/* -- Legal Info --
 * This code is Public Domain.
 */

// Include Files
#include "org.h"
#include "generic.cpp"

namespace Org
{
	FMOD_SYSTEM *Device;
	FMOD_CHANNELGROUP *MasterCG;
	void Sampler::SetMasterVolume(float p_Volume)
	{
		FMOD_RESULT l_Result = FMOD_ChannelGroup_SetVolume(Org::MasterCG, p_Volume);
		FmodErrorCheck(l_Result);
	}
	
	Sampler::Sampler()
	{
		// Melody
		{
			string l_Melodyprefix("melody/melody");
			string l_Suffix(".wav");
			for(int i=0; i<100; i++)
			{
				melodysamples[i] = FMOD_LoadSample(l_Melodyprefix + toString(i) + l_Suffix,1);
			}
		}
		
		// Percussion
		{
			percussionsamples[ 0] = FMOD_LoadSample("percussion/Bass01.wav",0);
			percussionsamples[ 1] = FMOD_LoadSample("percussion/Bass02.wav",0);
			percussionsamples[ 2] = FMOD_LoadSample("percussion/Snare01.wav",0);
			percussionsamples[ 3] = FMOD_LoadSample("percussion/Snare02.wav",0);
			percussionsamples[ 4] = FMOD_LoadSample("percussion/Tom01.wav",0);
			percussionsamples[ 5] = FMOD_LoadSample("percussion/HiClose.wav",0);
			percussionsamples[ 6] = FMOD_LoadSample("percussion/HiOpen.wav",0);
			percussionsamples[ 7] = FMOD_LoadSample("percussion/Crash.wav",0);
			percussionsamples[ 8] = FMOD_LoadSample("percussion/Per01.wav",0);
			percussionsamples[ 9] = FMOD_LoadSample("percussion/Per02.wav",0);
			percussionsamples[10] = FMOD_LoadSample("percussion/Bass03.wav",0);
			percussionsamples[11] = FMOD_LoadSample("percussion/Tom02.wav",0);
		}

		// Clear Junk
		for(int i=0; i<16; i++)
		{
			channellength[i] = 0;
			channelindex[i] = 0;
		}
		
		// Calculate Pitch Scale
		for(int i = 0; i < 12*8; i++)
		{
			pitch[i] = (int)(441.0*(pow(2.0,((i-19.0)/12.0))));
		}
	}
	bool Sampler::HitNote(SampleData notehit)
	{
		// Frequency calculation. The multiplier on the end, btw, is 44100/1550. Its just a randomly tweaked value.
		double freq = ((notehit.pitch - 1000.0)/100.0 + pitch[notehit.note])*44100/1550;
		
		// Set the sample.
		channelsample[notehit.instrument] = notehit.instrument<8?melodysamples[notehit.sample]:percussionsamples[notehit.sample];
		
		// Needed because indivisual notes do not loop. Don't ask...
		if(notehit.playing == 1) FMOD_StopSound(&channelindex[notehit.instrument]);
		
		// Do not remove, this case occurs for some reason.
		if(notehit.note > 12*8) return false;
		
		// Play the sound and save the handle into channelindex[]
		channelindex[notehit.instrument] = FMOD_PlaySound(channelsample[notehit.instrument], freq, notehit.pan, notehit.volume, channelindex[notehit.instrument], true);
		
		return true;
	}
	bool Sampler::Update(int ms)
	{
		// This is all we have to do with our slightly more CPU intensive solution to playing back ORG files.
		FMOD_System_Update(Org::Device);
	}
	OrgSong::OrgSong()
	{
		// Always blank out variables - Especially important WHENCE:
		// - Using ORGestrate for minipulation
		// - Something happens and loading does not occur, when the file is loaded manually or seperately.
		LoadBlank();
	}
	OrgSong::OrgSong(string filename)
	{
		// Simple wrapper to make life easier. Though really you could just do new OrgSong().LoadFromFile(filename) either way...
		LoadFromFile(filename);
	}
	bool OrgSong::LoadBlank()
	{
		// Blank out structs.
		memset(&header    ,0,sizeof(Header                   ));
		memset(instruments,0,sizeof(Instrument           [16]));
		memset(notepos    ,0,sizeof(vector<unsigned int> [16]));
		memset(notenote   ,0,sizeof(vector<unsigned int> [16]));
		memset(notelength ,0,sizeof(vector<unsigned int> [16]));
		memset(notevolume ,0,sizeof(vector<unsigned int> [16]));
		memset(notepan    ,0,sizeof(vector<unsigned int> [16]));

		// Set defaults. Everything must be blanked.
		cursor		= 0;
		isPlaying	= false;
		useLoopPoints	= false;
		msSinceBeat	= 0;
		msLastUpdate	= 0;
		cursorUpdated	= true;
		
		// This stuff has extra calculation, to avoid duplicate code we do these outside.
		SetTempo(140);
		SetSPBABPS(_4_4);
		SetLoopStart(0);
		SetLoopStop(0);
	}
	bool OrgSong::LoadFromFile(string filename)
	{
		// Local Variables.
		char hstring[7] = "      ";	// Has a default so it will not be filled with junk bytes.
		int length = 6 + 12 + 6 * 16;	// Size of all the headers.
		
		// Blank out class variables.
		LoadBlank();
		
		// Load filename.
		ifstream file(filename.c_str(), ios::binary);
		file.read((char *)&hstring, sizeof(char[6]));
		file.read((char *)&header, sizeof(char[12]));
		
		// Force full update
		SetTempo(header.tempo);
		
		file.read((char *)instruments, sizeof(char[6][16]));
		
		for(int i=0;i<16;i++)
		{
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned int tpos;
				file.read((char *)&tpos, sizeof(unsigned int));
				notepos[i].push_back(tpos);
				length += 4;
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tnote;
				file.read((char *)&tnote, sizeof(char[1]));
				notenote[i].push_back(tnote);
				length += 1;
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tlength;
				file.read((char *)&tlength, sizeof(char[1]));
				notelength[i].push_back(tlength);
				length += 1;
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tvolume;
				file.read((char *)&tvolume, sizeof(char[1]));
				notevolume[i].push_back(tvolume);
				length += 1;
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tpan;
				file.read((char *)&tpan, sizeof(char[1]));
				notepan[i].push_back(tpan);
				length += 1;
			}
		}
		std::cerr << "Read in " << length << " bytes.\n";
	}
	bool OrgSong::SaveToFile(string filename)
	{
		// Local Variables
		char hstring[7] = "Org-02";
		
		// Blank out class variables.
		LoadBlank();
		
		// Load filename.
		ofstream file(filename.c_str(), ios::binary);
		file.write((char *)&hstring, sizeof(char[6]));
		file.write((char *)&header, sizeof(char[12]));
		
		// Force full update
		SetTempo(header.tempo);
		
		file.write((char *)instruments, sizeof(char[6][16]));
		
		for(int i=0;i<16;i++)
		{
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned int tpos = notepos[i][x];
				file.write((char *)&tpos, sizeof(unsigned int));
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tnote = notenote[i][x];
				file.write((char *)&tnote, sizeof(char[1]));
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tlength = notelength[i][x];
				file.write((char *)&tlength, sizeof(char[1]));
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tvolume = notevolume[i][x];
				file.write((char *)&tvolume, sizeof(char[1]));
			}
			for(int x=0;x<instruments[i].numnotes;x++)
			{
				unsigned char tpan = notepan[i][x];
				file.write((char *)&tpan, sizeof(char[1]));
			}
		}
	}
	int OrgSong::Update()
	{
		// Gets the time in milliseconds
		unsigned long time = SDL_GetTicks();
		unsigned long dif = time-msLastUpdate;
		
		// We have to had updated before for this to work.
		if(msLastUpdate)
		{
			if(isPlaying && dif)
			{
				int oldcursor = cursor;
				cursor += (double)dif/(double)msPerBeat;
				cursorUpdated = false;
				if((int)cursor != (int)oldcursor)
				{
					cursorUpdated = true;
					if(cursor >= header.loopstop)
					{
						cursor = header.loopstart;
					}
				}
			}
		}
		
		// Set the clock even in cases of failure.
		msLastUpdate = time;
		
		return dif;
	}
	
	bool OrgSong::IsPlaying()
	{
		return isPlaying;
	}
	
	bool OrgSong::Play()
	{
		if(isPlaying)
		{
			isPlaying = false;
		}
		else
		{
			isPlaying = true;
		}
		
		return isPlaying;
	}
	unsigned int OrgSong::GetCursor()
	{
		return cursor;
	}
	//           Cursor = 13
	//               V
	//  BBBBBNnnnnBBBN
	//  ^=0  ^1^3^5^5^6
	SampleData OrgSong::GetNoteHit(int instrument)
	{
		return GetNoteHit(instrument, cursor);
	}
	SampleData OrgSong::GetNoteHit(int instrument, int pcursor)
	{
		SampleData sd;
		memset(&sd,0,sizeof(SampleData));
		sd.instrument = instrument;
		for(int i=0; i<instruments[instrument].numnotes; i++)
		{
			if(notepos[instrument][i] == (int)pcursor)
			{
				if(notelength[instrument][i])
				{
					sd.note = notenote[instrument][i];
					sd.length = notelength[instrument][i];
					sd.volume = notevolume[instrument][i];
					if(notepan[instrument][i] != 255)
					{
						sd.pan = (((double)notepan[instrument][i]/6.0)-1.0);
					}
					else
					{
						sd.pan = 0;
					}
					sd.pitch = instruments[instrument].pitch;
					sd.sample = instruments[instrument].id;
					sd.loop = !instruments[instrument].noloop;
					sd.playing = 1;
					return sd;
				}
			}

			if(notepos[instrument][i] < (int)pcursor)
			{
				if(notepos[instrument][i]+notelength[instrument][i] > (int)pcursor)
				{
					// Because its only possible to have notelength-1 modifiers
					for(int x=i+notelength[instrument][i]; x>i; x--)
					{
						if(notepos[instrument][x] <= (int)pcursor && x<instruments[instrument].numnotes)
						{
							sd.note = notenote[instrument][i];
							sd.length = notepos[instrument][i]+notelength[instrument][i]-pcursor;
							sd.volume = notevolume[instrument][x];
							if(notepan[instrument][x] != 255)
							{
								sd.pan = (((double)notepan[instrument][x]/6.0)-1.0);
							}
							else
							{
								if(notepan[instrument][i] != 255)
								{
									sd.pan = (((double)notepan[instrument][i]/6.0)-1.0);
								}
								else
								{
									sd.pan = 0;
								}
							}
							sd.pitch = instruments[instrument].pitch;
							sd.sample = instruments[instrument].id;
							sd.loop = !instruments[instrument].noloop;
							sd.playing = 2;
							return sd;
						}
					}
					sd.note = notenote[instrument][i];
					sd.length = 1;
					sd.volume = notevolume[instrument][i];
					if(notepan[instrument][i] != 255)
					{
						sd.pan = (((double)notepan[instrument][i]/6.0)-1.0);
					}
					else
					{
						sd.pan = 0;
					}
					sd.pitch = instruments[instrument].pitch;
					sd.sample = instruments[instrument].id;
					sd.loop = !instruments[instrument].noloop;
					sd.playing = 2;
					return sd;
				}
			}
		}

		return sd;
	}
	bool OrgSong::SetTempo(unsigned short to)
	{
		header.tempo = to;
		msPerBeat = (long double)header.tempo;
		
		return true;
	}
	bool OrgSong::SetSPBABPS(SPBABPS to)
	{
		switch(to)
		{
			case _2_4: header.steps_bar=2; header.beats_step=4; break;
			case _3_3: header.steps_bar=3; header.beats_step=3; break;
			case _3_4: header.steps_bar=3; header.beats_step=4; break;
			case _4_3: header.steps_bar=4; header.beats_step=3; break;
			case _4_4: header.steps_bar=4; header.beats_step=4; break;
			case _6_3: header.steps_bar=6; header.beats_step=3; break;
			case _6_4: header.steps_bar=6; header.beats_step=4; break;
			case _8_4: header.steps_bar=8; header.beats_step=4; break;
			default: fprintf(stderr,"Error: Unknown SPBABPS value.\n"); return false;
		}
		return true;
	}
	bool OrgSong::SetLoopStart(unsigned int to)
	{
		header.loopstart = to;
		
		return true;
	}

	bool OrgSong::SetLoopStop(unsigned int to)
	{
		header.loopstop = to;
		
		return true;
	}
}

