/* -- File Info --
 * orgGL.h - Header file
 */

/* -- Legal Info --
 * This code is Public Domain.
 */

#ifndef ORGGL_H
#define ORGGL_H

#include "orggui.h"
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <SDL/SDL_image.h>
#include <FTGL/FTGLTextureFont.h>

class GraphicsOpenGL : public GraphicsOutput
{
	public:
		// Synopsis:
		//  Init(int p_Width, int p_Height, int p_PreferedBPP)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Width: Width the output screen should be.
		//  p_Height: Height the output screen should be.
		//  p_PreferedBPP: The BPP we want to have.
		bool Init(int p_Width, int p_Height, int p_PreferedBPP);
		
		// Synopsis:
		//  LoadTexture(void *p_BitmapData, unsigned int width, unsigned int height)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Surface: Pointer to the SDL Surface
		// Returns:
		//  The return data should be a pointer to implementation-specific texture data (Or a struct to hold that stuff)
		void *LoadTexture(SDL_Surface *p_Surface);
		
		// Synopsis:
		//  DrawRect(void *p_Texture, TextureCoord p_TextureCoordinates, RectangleShape p_DrawRect)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Texture: Pointer to the return data of a LoadTexture call.
		//  p_TextureCoordinates: Normalized texture coordinates. (0.0, 0.0) -> (1.0, 1.0) would show the whole image. 
		//  p_DrawRect: The area the rectange actually lies.
		void DrawRect(void *p_Texture, TextureCoord p_TextureCoordinates, RectangleShape p_DrawRect, float p_Color[4]);
		
		// Synopsis:
		//  DrawText(wchar_t *p_String, bool p_IsLarge, Position p_Point)
		// Description:
		//  Draws a font that can consist of two sizes, Small and Large (12 pt, 24 pt)
		// Parameters:
		//  p_String: a UTF-16 compliant string to draw.
		//  p_IsLarge: If true the font will be drawn at 24 pt, if not 12 pt.
		//  p_Point: The actual position the text is to be drawn.
		void DrawText(wchar_t *p_String, bool p_IsLarge, Position p_Point, Orientation p_Orientation);
		
		// Synopsis:
		//  Update()
		// Description:
		//  Updates the graphics, provided they don't update immediately.
		void Update();
		
		// Synopsis:
		//  Close()
		// Description:
		//  Closes a graphics output device.
		void Close();
	private:
		SDL_Surface *screen;
		FTGLTextureFont *fontsmall, *fontlarge;
};

#endif
