/* -- File Info --
 * orggui.h - Header file
 */

/* -- Legal Info --
 * This code is Public Domain.
 */

#ifndef ORGGUI_H
#define ORGGUI_H

#include "org.h"
#include "generic.cpp"

enum Orientation
{
	FONT_LEFT,
	FONT_CENTER,
	FONT_RIGHT
};

// Simple class. Doesn't need as many functions as a geometric Rectangle class should have.
class TextureCoord
{
	public:
		double X1, Y1, X2, Y2;
		
		TextureCoord();
		TextureCoord(double p_Scale);
		TextureCoord(double p_W, double p_H);
		TextureCoord(double p_X1, double p_Y1, double p_X2, double p_Y2);
		
		void Offset(double p_X, double p_Y);
		
	private:
};

class GraphicsOutput
{
	public:
		// Synopsis:
		//  Init(int p_Width, int p_Height, int p_PreferedBPP)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Width: Width the output screen should be.
		//  p_Height: Height the output screen should be.
		//  p_PreferedBPP: The BPP we want to have.
		virtual bool Init(int p_Width, int p_Height, int p_PreferedBPP) { return false; };
		
		// Synopsis:
		//  LoadTexture(void *p_BitmapData, unsigned int width, unsigned int height)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Surface: Pointer to the SDL Surface
		// Returns:
		//  The return data should be a pointer to implementation-specific texture data (Or a struct to hold that stuff)
		virtual void *LoadTexture(SDL_Surface *p_Surface) { return NULL; };
		
		// Synopsis:
		//  DrawRect(void *p_Texture, TextureCoord p_TextureCoordinates, RectangleShape p_DrawRect)
		// Description:
		//  Initalize a graphics output device.
		// Parameters:
		//  p_Texture: Pointer to the return data of a LoadTexture call.
		//  p_TextureCoordinates: Normalized texture coordinates. (0.0, 0.0) -> (1.0, 1.0) would show the whole image. 
		//  p_DrawRect: The area the rectange actually lies.
		virtual void DrawRect(void *p_Texture, TextureCoord p_TextureCoordinates, RectangleShape p_DrawRect, float p_Color[4]) { return; };
		
		// Synopsis:
		//  DrawText(wchar_t *p_String, bool p_IsLarge, Position p_Point)
		// Description:
		//  Draws a font that can consist of two sizes, Small and Large (12 pt, 24 pt)
		// Parameters:
		//  p_String: a UTF-16 compliant string to draw.
		//  p_IsLarge: If true the font will be drawn at 24 pt, if not 12 pt.
		//  p_Point: The actual position the text is to be drawn.
		virtual void DrawText(wchar_t *p_String, bool p_IsLarge, Position p_Point, Orientation p_Orientation) { return; }
		
		// Synopsis:
		//  Update()
		// Description:
		//  Updates the graphics, provided they don't update immediately.
		virtual void Update() { return; };
		
		// Synopsis:
		//  Close()
		// Description:
		//  Closes a graphics output device.
		virtual void Close() { return; };
	private:
};

extern GraphicsOutput *g_GfxOut;
#include "orgtheme.h"

class Widget
{
	public:
		virtual bool Draw() = 0;
		virtual bool RegisterMouseDown() = 0;
		virtual bool RegisterMouseUp() = 0;
		virtual bool RegisterMouseMotion() = 0;
		
		bool GetDimensions(float *p_X, float *p_Y, float *p_W, float *p_H)
		{
			if(p_X) *p_X = X;
			if(p_Y) *p_Y = Y;
			if(p_W) *p_W = W;
			if(p_H) *p_H = H;
			
			return true;
		}
		bool SetDimensions(float p_X, float p_Y, float p_W, float p_H)
		{
			X = p_X;
			Y = p_Y;
			W = p_W;
			H = p_H;
			
			return true;
		}
		bool SetDimensionX(float p_X)
		{
			X = p_X;
			
			return true;
		}
		bool SetDimensionY(float p_Y)
		{
			Y = p_Y;
			
			return true;
		}
		bool SetDimensionW(float p_W)
		{
			W = p_W;
			
			return true;
		}
		bool SetDimensionH(float p_H)
		{
			H = p_H;
			
			return true;
		}
		
		float X, Y, W, H;
		float Color[4];
};

typedef void (* RegVolume)(float Current);

class VolumeSlider : public Widget
{
	public:
		VolumeSlider(float p_Min, float p_Max, float p_Current, RegVolume p_Callback)
		{
			Color[0] = 1.0;
			Color[1] = 1.0;
			Color[2] = 1.0;
			Color[3] = 1.0;
			
			X = 0;
			Y = 0;
			W = 175;
			H = 100;
			
			Min = p_Min;
			Max = p_Max;
			Current = p_Current;
			Callback = p_Callback;
			Pressed = false;
		}
		bool Draw()
		{
			g_GfxOut->DrawRect(g_ThemeBitmaps[VOLUMESLIDER], TextureCoord(0,0,((Current-Min)/(Max-Min))*(175.0/200.0),1), RectangleShape(X,Y,(Current-Min)*(W/(Max-Min)),H), Color);
			g_GfxOut->DrawRect(g_ThemeBitmaps[VOLUMESLIDER], TextureCoord(175.0/200.0,0,1,1), RectangleShape(X+Current*(W/(Max-Min))-10,Y,20,H), Color);
		}
		bool RegisterMouseDown()
		{
			if( SDL_GetMouseState(&DragX, &DragY) & SDL_BUTTON(SDL_BUTTON_LEFT))
			{
				if(DragX > ((Current-Min)*(W/(Max-Min)))+X-10 && DragX < ((Current-Min)*(W/(Max-Min)))+X+10 && DragY > Y && DragY < Y+H)
				{
					DragX -= (Current-Min)*(W/Max);
					Pressed = true;
					return true;
				}
				else
				{
					Pressed = false;
					return false;
				}
			}
		}
		bool RegisterMouseMotion()
		{
			if(Pressed)
			{
				int l_MouseX, l_MouseY;
				SDL_GetMouseState(&l_MouseX, &l_MouseY);
				
				Current = ((float)l_MouseX-DragX)/(W/(Max-Min))+Min;
				if(Current > Max)
				{
					Current = Max;
				}	
				if(Current < Min)
				{
					Current = Min;
				}
				Callback(Current);
				
				return true;
			}
			return false;
		}
		bool RegisterMouseUp()
		{
			Pressed = false;
			
			return true;
		}
		bool GetMin(float *p_Min)
		{
			if(p_Min) 
			{
				*p_Min = Min;
				return true;
			}
			return false;
		}
		bool GetMax(float *p_Max)
		{
			if(p_Max) 
			{
				*p_Max = Max;
				return true;
			}
			return false;
		}
		bool GetCurrent(float *p_Current)
		{
			if(p_Current) 
			{
				*p_Current = Current;
				return true;
			}
			return false;
		}
		bool SetMin(float p_Min)
		{
			Min = p_Min;
			return true;
		}
		bool SetMax(float p_Max)
		{
			Max = p_Max;
			return true;
		}
		bool SetCurrent(float p_Current)
		{
			Current = p_Current;
			return true;
		}
		
	private:
		float Min,Max,Current; 
		int DragX, DragY;
		bool Pressed;
		RegVolume Callback;
};

#endif
