/* -- File Info --
 * orgplay.cpp - Implementation file
 *
 * Simply a public domain ORG player.
 * Do what you wish with it. Providing credit would be cool but I just don't see many people wanting to do that so meh.
 *
 * To note, this code is PD. You may copy it, redistribute it, and use it as you wish, unrestricted.
 *   ~nmn
 */

/* -- Legal Info --
 * This code is Public Domain.
 */

#include "orgplay.h"
#include "orgtheme.h"

float g_White[4] = { 1.0f, 1.0f, 1.0f, 1.0f };

wchar_t *Expand(char* p_String)
{
	wchar_t *p_UnicodeString = new wchar_t[sizeof(wchar_t[strlen(p_String)+1])];
	memset(p_UnicodeString,0,strlen(p_String)*2);
	for(int i=0; i < strlen(p_String); i++)
	{
		wchar_t l_SingleUniChar = (wchar_t)p_String[i];
		p_UnicodeString[i] = l_SingleUniChar;
	}
	p_UnicodeString[strlen(p_String)] = L'\0';
	return p_UnicodeString;
}

Org::Sampler *sampler;
void VolumeRegister(float Current)
{
	sampler->SetMasterVolume(Current);
}
#ifndef __WIN32__
#define _main main
#endif

int _main(int argc, char **argv)
{
	// Local Variables
	SDL_Event l_Event;
	bool l_Hold;
	float l_Volume = 0.5;
	vector<Widget *> l_WidgetList;
	wchar_t* music = Expand(argv[argc-1]);
	
	// Set up volume slider
	l_WidgetList.push_back(new VolumeSlider(0.0,1.0,0.5,VolumeRegister));
	l_WidgetList[l_WidgetList.size()-1]->SetDimensions(45,235,200,50);
	
	// Initalize SDL
	SDL_Init(0);
	
	// Initalize the graphics engine
	g_GfxOut = new GraphicsOpenGL();
	g_GfxOut->Init(300,350,32);
	
	// Iterate themes and set to first found
	GetThemes();
	SetTheme(0, g_GfxOut);
	
	// Initalize FMOD
	InitFMOD();
	
	// Create a new Sampler.
	sampler = new Org::Sampler();
		
	// Open a song.
	Org::OrgSong song(argv[argc-1]); song.Play();
	
	//fprintf(stderr,"Entering main loop.\n");
	
	// Set default volume
	sampler->SetMasterVolume(l_Volume);
	
	while(1)
	{
		// First we update the song and sampler.
		if(song.cursorUpdated)
		{
			Org::SampleData sd;
			for(int i=0; i<16; i++)
			{
				sd = song.GetNoteHit(i);  sampler->HitNote(sd);
			}
		}
		sampler->Update(song.Update());
		
		// Next we enter our SDL event loop.
		while(SDL_PollEvent(&l_Event)) 
		{
			switch(l_Event.type) 
			{
				case SDL_KEYDOWN:
				break;
				
				case SDL_MOUSEMOTION:
				{
					for(int i=0; i<l_WidgetList.size(); i++)
					{
						l_WidgetList[i]->RegisterMouseMotion();
					}
				}
				break;
					
				case SDL_MOUSEBUTTONDOWN:
				{
					for(int i=0; i<l_WidgetList.size(); i++)
					{
						l_WidgetList[i]->RegisterMouseDown();
					}
				}
				break;
					
				case SDL_MOUSEBUTTONUP:
				{
					for(int i=0; i<l_WidgetList.size(); i++)
					{
						l_WidgetList[i]->RegisterMouseUp();
					}
				}
				break;
					
				case SDL_QUIT:
				{
					exit(0);
				}
				break;
				
				default:
				break;
			} 
			
		}
		
		g_GfxOut->DrawRect(g_ThemeBitmaps[BACKGROUND], TextureCoord(), RectangleShape(300,350), g_White);
		g_GfxOut->DrawText(music,false,Position(150,100),FONT_CENTER);
		for(int i=0; i<l_WidgetList.size(); i++)
		{
			l_WidgetList[i]->Draw();
		}
		
		// Thirdly we update the graphics.
		g_GfxOut->Update();
		
		// Forth and lastly we pause for 10000 nano-seconds (10ms)
		//usleep(10000);
	}
}
